import javax.xml.transform.TransformerException;

import org.apache.xpath.XPathAPI;

import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.NodeList;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSParser;
import org.w3c.dom.ls.LSSerializer;
import org.w3c.dom.ranges.DocumentRange;
import org.w3c.dom.ranges.Range;

/**
 * This class provides demonstrates the DOM Range interface and 
 * the Load and Save Serializer Interface
 * 
 * The XPath API was taken from Xalan since it didn't work yet.
 * 
 * @author Udo Altmann
 * @version 1.0 (2005)
 * 
 */
public class RangeDemo {

	/**
	 * @param args xmlfile xpath1 xpath2
	 * @throws ClassCastException
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws TransformerException
	 * 
	 */
	public static void main(String[] args) throws ClassCastException, ClassNotFoundException, InstantiationException, IllegalAccessException, TransformerException {
		
		if(args.length< 3){
			System.out.println("RangeDemo xmlfile xpath1 xpath2");
			return;
		}
		System.setProperty(DOMImplementationRegistry.PROPERTY,
				"org.apache.xerces.dom.DOMImplementationSourceImpl");

		// Class from org.w3c.dom.bootstrap
		DOMImplementationRegistry registry;
		registry = DOMImplementationRegistry.newInstance();

		// Require Load/Save and Range
		DOMImplementationLS impl = (DOMImplementationLS) registry
				.getDOMImplementation("LS Range"); 

		LSParser parser = impl.createLSParser(
				DOMImplementationLS.MODE_SYNCHRONOUS, null);

		DOMConfiguration conf = parser.getDomConfig();

		// Set Parameters
		conf.setParameter("validate-if-schema", new Boolean(true));
		conf.setParameter("cdata-sections", new Boolean(false));
		conf.setParameter("entities", new Boolean(false));
		Document doc = parser.parseURI(args[0]);

		if(((DOMImplementation)impl).hasFeature("Range", "2.0")){
			// Interface is fetched from the Document object 
			DocumentRange dr = (DocumentRange)doc;
			Range r = dr.createRange();
			
			NodeList nl; 

			// We don't care about the length of the NodeList, demo only use
			// Set start and end
			// The XPathAPI is a Non-DOM Xalan class 
			nl = XPathAPI.selectNodeList(doc, args[1]);
			r.setStart(nl.item(0), 0);
			
			nl = XPathAPI.selectNodeList(doc, args[2]);
			r.setEnd(nl.item(0), 0);
			
			// Create a DocumentFragment
			DocumentFragment df = r.cloneContents();
			
			// Create a Serializer - see createLSParser 
			LSSerializer writer = impl.createLSSerializer();
			System.out.println(writer.writeToString(df));
		} else {
			System.out.println("Range not supported");
		}

	}

}
